@@ -48,7 +48,7 @@ impl Credential {
         // Take 120s as buffer to avoid edge cases.
         if let Some(valid) = self
             .expires_in
-            .map(|v| v > now() + chrono::Duration::minutes(2))
+            .map(|v| v > now() + chrono::TimeDelta::try_minutes(2).expect("in bounds"))
         {
             return valid;
         }
@@ -117,34 +117,34 @@ impl DefaultLoader {
         let cred = self.load_inner().await?;
 
         let mut lock = self.credential.lock().expect("lock poisoned");
-        *lock = cred.clone();
+        lock.clone_from(&cred);
 
         Ok(cred)
     }
 
     async fn load_inner(&self) -> Result<Option<Credential>> {
-        if let Ok(Some(cred)) = self
-            .load_via_config()
-            .map_err(|err| debug!("load credential via config failed: {err:?}"))
-        {
+        if let Some(cred) = self.load_via_config().map_err(|err| {
+            debug!("load credential via config failed: {err:?}");
+            err
+        })? {
             return Ok(Some(cred));
         }
 
-        if let Ok(Some(cred)) = self
+        if let Some(cred) = self
             .load_via_assume_role_with_web_identity()
             .await
             .map_err(|err| {
-                debug!("load credential via assume_role_with_web_identity failed: {err:?}")
-            })
+                debug!("load credential via assume_role_with_web_identity failed: {err:?}");
+                err
+            })?
         {
             return Ok(Some(cred));
         }
 
-        if let Ok(Some(cred)) = self
-            .load_via_imds_v2()
-            .await
-            .map_err(|err| debug!("load credential via imds_v2 failed: {err:?}"))
-        {
+        if let Some(cred) = self.load_via_imds_v2().await.map_err(|err| {
+            debug!("load credential via imds_v2 failed: {err:?}");
+            err
+        })? {
             return Ok(Some(cred));
         }
 
@@ -159,7 +159,7 @@ impl DefaultLoader {
                 session_token: self.config.session_token.clone(),
                 // Set expires_in to 10 minutes to enforce re-read
                 // from file.
-                expires_in: Some(now() + chrono::Duration::minutes(10)),
+                expires_in: Some(now() + chrono::TimeDelta::try_minutes(10).expect("in bounds")),
             }))
         } else {
             Ok(None)
@@ -356,7 +356,8 @@ impl IMDSv2Loader {
         }
         let ec2_token = resp.text().await?;
         // Set expires_in to 10 minutes to enforce re-read.
-        let expires_in = now() + chrono::Duration::seconds(21600) - chrono::Duration::seconds(600);
+        let expires_in = now() + chrono::TimeDelta::try_seconds(21600).expect("in bounds")
+            - chrono::TimeDelta::try_seconds(600).expect("in bounds");
 
         {
             *self.token.lock().expect("lock poisoned") = (ec2_token.clone(), expires_in);
@@ -418,6 +419,19 @@ impl AssumeRoleLoader {
         if let Some(external_id) = &self.config.external_id {
             write!(url, "&ExternalId={external_id}")?;
         }
+        if let Some(duration_seconds) = &self.config.duration_seconds {
+            write!(url, "&DurationSeconds={duration_seconds}")?;
+        }
+        if let Some(tags) = &self.config.tags {
+            for (idx, (key, value)) in tags.iter().enumerate() {
+                let tag_index = idx + 1;
+                write!(
+                    url,
+                    "&Tags.member.{tag_index}.Key={key}&Tags.member.{tag_index}.Value={value}"
+                )?;
+            }
+        }
+
         let mut req = self
             .client
             .get(&url)
